@@ -2,20 +2,19 @@
 # this repository contains the full copyright notices and license terms.
 
 from trytond.pool import Pool
-from stock import *
-from production import *
-from product import *
+
+from . import product, production, stock
 
 
 def register():
     Pool.register(
-        OrderPoint,
-        LocationLeadTime,
-        Configuration,
-        ConfigurationSupplyPeriod,
-        Production,
-        Product,
+        stock.OrderPoint,
+        stock.LocationLeadTime,
+        production.Configuration,
+        production.ConfigurationSupplyPeriod,
+        production.Production,
+        product.Product,
         module='stock_supply_production', type_='model')
     Pool.register(
-        StockSupply,
+        stock.StockSupply,
         module='stock_supply_production', type_='wizard')
